//  Anpassung an Arduino Digispark ATtiny85                                  JoFri 2024-03-14
//  Die Funktionstasten F0, F7, F8, F9 steuern die Ausgänge PB0, PB1, PB3, PB4 für die LED's 
//  Pin PB5 nicht verwendet, wenn an Pin5 was angesteckt wird , erfolgt ein Reset    
//
//
//    Board Auswahl: 
//      alt              Digispark (Default - 16.5mhz)
//      neu  ATTinyCore  ATtiny85 (Mironucleus / Digispark)
//
//
//  benötigt wird die NmraDcc-master.zip  <NmraDcc.h> 
//  Diese Bibliothek ist auf neuerem Stand unter:  https://github.com/mrrwa/NmraDcc
//  hier gibst auch ältere Versionen https://reference.arduino.cc/reference/en/libraries/nmradcc/  
//  es wird die <NmraDcc.h> in der Version 1.4.2 benötigt, neuere Versionen funktionieren nicht mehr.
//
// Ursprüngliche Quelle:
// Production 17 Function DCC Decoder   Dec_17LED_1Ftn.ino
// Version 6.01  Geoff Bunza 2014,2015,2016,2017,2018
// Now works with both short and long DCC Addesses
// ******** UNLESS YOU WANT ALL CV'S RESET UPON EVERY POWER UP
// ******** AFTER THE INITIAL DECODER LOAD REMOVE THE "//" IN THE FOOLOWING LINE!!
// #define DECODER_LOADED
//

/*
   Arduino ATtiny85                                 
              +--------------------+                
              |                    |   Pin             
         |-----                [ ] |    5  PB5  F4  (Reset ? frei)
         | [-]    ATtiny85     [ ] |    4  PB4  F3  
    USB  | [ ]                 [ ] |    3  PB3  F2  
         | [ ]                 [ ] |    2  PB2  DCC Signal Eingang 
         | [+]                 [ ] |    1  PB1  F1  
         |-----                [ ] |    0  PB0  F0  
              |  [ ] [ ] [ ]    [+]|                
              +--------------------+                
                +5V GND  Vin                        
*/


#include <NmraDcc.h>

#define This_Decoder_Address 22  // CV1 DCC Adresse des Decoder

const int FunctionPin0 = 0;      // PB0 F0
const int FunctionPin1 = 1;      // PB1 F1          interne LED
const int FunctionPin2 = 3;      // PB3 F2      
const int FunctionPin3 = 4;      // PB4 F3
//const int FunctionPin4 = 5;      // PB5 F4     je nach Tiny85 Model ist dieser Ausgang ein Reset 


NmraDcc Dcc;
DCC_MSG Packet;
uint8_t CV_DECODER_MASTER_RESET = 120;

struct CVPair {
  uint16_t CV;
  uint8_t Value;
};

CVPair FactoryDefaultCVs[] = {
  { CV_MULTIFUNCTION_PRIMARY_ADDRESS, This_Decoder_Address & 0x7F },

  // These two CVs define the Long DCC Address
  { CV_MULTIFUNCTION_EXTENDED_ADDRESS_MSB, ((This_Decoder_Address >> 8) & 0x7F) + 192 },
  { CV_MULTIFUNCTION_EXTENDED_ADDRESS_LSB, This_Decoder_Address & 0xFF },

  // ONLY uncomment 1 CV_29_CONFIG line below as approprate DEFAULT IS SHORT ADDRESS
  //  {CV_29_CONFIG,          0},                               // Short Address 14 Speed Steps
  { CV_29_CONFIG, CV29_F0_LOCATION },                           // Short Address 28/128 Speed Steps
  //  {CV_29_CONFIG, CV29_EXT_ADDRESSING | CV29_F0_LOCATION},   // Long  Address 28/128 Speed Steps

  { CV_DECODER_MASTER_RESET, 0 },
};

uint8_t FactoryDefaultCVIndex = sizeof(FactoryDefaultCVs) / sizeof(CVPair);

void notifyCVResetFactoryDefault() {
  // Make FactoryDefaultCVIndex non-zero and equal to num CV's to be reset
  // to flag to the loop() function that a reset to Factory Defaults needs to be done
  FactoryDefaultCVIndex = sizeof(FactoryDefaultCVs) / sizeof(CVPair);
};

void setup() {

  // initialize the digital pins as an outputs
    pinMode(0, OUTPUT);
    pinMode(1, OUTPUT);
    pinMode(2, INPUT);     // DCC Eingang
    pinMode(3, OUTPUT);
    pinMode(4, OUTPUT);    
//    pinMode(5, OUTPUT);   // Reset 

    digitalWrite(0, LOW); 
    digitalWrite(1, LOW); 
    digitalWrite(3, LOW); 
    digitalWrite(4, LOW); 
//    digitalWrite(5, LOW);  // Reset 


  // Setup which External Interrupt, the Pin it's associated with that we're using and enable the Pull-Up
  Dcc.pin(0, 2, 0);
  // Call the main DCC Init function to enable the DCC Receiver
  Dcc.init(MAN_ID_DIY, 601, FLAGS_MY_ADDRESS_ONLY, 0);
  delay(800);
#if defined(DECODER_LOADED)
  if (Dcc.getCV(CV_DECODER_MASTER_RESET) == CV_DECODER_MASTER_RESET)
#endif
  {
    for (int j = 0; j < FactoryDefaultCVIndex; j++)
      Dcc.setCV(FactoryDefaultCVs[j].CV, FactoryDefaultCVs[j].Value);
    digitalWrite(13, 1);  //Blink the on board LED
    delay(1000);
    digitalWrite(13, 0);
  }
}

void loop() {
  // You MUST call the NmraDcc.process() method frequently from the Arduino loop() function for correct library operation
  Dcc.process();
  if (FactoryDefaultCVIndex && Dcc.isSetCVReady()) {
    FactoryDefaultCVIndex--;  // Decrement first as initially it is the size of the array
    Dcc.setCV(FactoryDefaultCVs[FactoryDefaultCVIndex].CV, FactoryDefaultCVs[FactoryDefaultCVIndex].Value);
  }
}

void notifyDccFunc(uint16_t Addr, DCC_ADDR_TYPE AddrType, FN_GROUP FuncGrp, uint8_t FuncState) {
  switch (FuncGrp) {
    case FN_0_4:
      digitalWrite(FunctionPin0, (FuncState & FN_BIT_00) >> 4);  // F0
//      digitalWrite(FunctionPin1, (FuncState & FN_BIT_01));       // F1
//      digitalWrite(FunctionPin2, (FuncState & FN_BIT_02) >> 1);  // F2
//      digitalWrite(FunctionPin3, (FuncState & FN_BIT_03) >> 2);  // F3
//      digitalWrite(FunctionPin3, (FuncState & FN_BIT_04) >> 3);  // F4
      break;

    case FN_5_8:
//      digitalWrite(FunctionPin5, (FuncState & FN_BIT_05));       // F5
//      digitalWrite(FunctionPin3, (FuncState & FN_BIT_06) >> 1);  // F6
      digitalWrite(FunctionPin1, (FuncState & FN_BIT_07) >> 2);  // F7
      digitalWrite(FunctionPin2, (FuncState & FN_BIT_08) >> 3);  // F8
      break;

    case FN_9_12:
      digitalWrite(FunctionPin3, (FuncState & FN_BIT_09));        // F9
//      digitalWrite(FunctionPin10, (FuncState & FN_BIT_10) >> 1);  // F10
//      digitalWrite(FunctionPin11, (FuncState & FN_BIT_11) >> 2);  // F11
//      digitalWrite(FunctionPin12, (FuncState & FN_BIT_12) >> 3);  // F12
      break;

    case FN_13_20:
//      digitalWrite(FunctionPin13, !((FuncState & FN_BIT_13)));       // F13     mit !(  xxx  ) invertiert  LOW = aktiv 
//      digitalWrite(FunctionPin14, !((FuncState & FN_BIT_14) >> 1));  // F14     mit !(  xxx  ) invertiert  LOW = aktiv
//      digitalWrite(FunctionPin15, (FuncState & FN_BIT_15) >> 2);     // F15        HIGH = aktiv
//      digitalWrite(FunctionPin16, (FuncState & FN_BIT_16) >> 3);     // F16        HIGH = aktiv
      break;

    case FN_21_28:

      break;
  }
}
